/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

import QtQuick 1.1
import com.nokia.meego 1.1
import com.nokia.extras 1.1
import EmuMaster 1.0

Page {
	id: settingsPage
	property alias columnContent: column.children
	property alias inputContent: inputColumn.children
	property alias videoContent: videoColumn.children
	property alias miscContent: miscColumn.children
	property string initiatedAction

	tools: ToolBarLayout {
		ToolIcon {
			iconId: "toolbar-back"
			onClicked: emuView.showEmulationView()
		}
		ToolIcon {
			iconId: "toolbar-add"
			onClicked: settingsPage.initAction("saveState")
		}
		ToolIcon {
			iconId: "toolbar-refresh"
			onClicked: settingsPage.initAction("emuReset")
		}
		ToolIcon {
			iconId: "toolbar-delete"
			onClicked: settingsPage.initAction("deleteAllStates")
		}
	}

	function yesNoDialogMessage(action) {
		switch (action) {
		case "overwriteState":	return qsTr("Do you really want to overwrite the saved state with the current one?")
		case "deleteState":		return qsTr("Do you really want to delete the saved state?")
		case "deleteAllStates":	return qsTr("Do you really want to delete all saved states?")
		case "emuReset":		return qsTr("Do you really want to restart the emulated system?")
		default:				return qsTr("Unknown action: "+action+", please send info to the developer!")
		}
	}

	function executeAction(action) {
		switch (action) {
		case "loadState":
			stateListModel.loadState(stateListView.selectedSlot);
			emuView.showEmulationView()
			break
		case "saveState":		stateListModel.saveState(/*StateListModel.NewSlot*/-1); break
		case "overwriteState":	stateListModel.saveState(stateListView.selectedSlot); break
		case "deleteState":		stateListModel.removeState(stateListView.selectedSlot); break
		case "deleteAllStates":	stateListModel.removeAll(); break
		case "emuReset":		emu.reset(); emuView.showEmulationView(); break
		default:				console.log("unknown action: " + action); break
		}
	}

	function initAction(action) {
		switch (action) {
		case "loadState":
		case "saveState":
			settingsPage.executeAction(action); break
		default:
			settingsPage.initiatedAction = action
			yesNoDialog.message = yesNoDialogMessage(action)
			yesNoDialog.open()
			break
		}
	}

	Menu {
		id: stateMenu

		MenuLayout {
			MenuItem { text: qsTr("Load");		onClicked: settingsPage.initAction("loadState") }
			MenuItem { text: qsTr("Overwrite");	onClicked: settingsPage.initAction("overwriteState") }
			MenuItem { text: qsTr("Delete");	onClicked: settingsPage.initAction("deleteState") }
		}
	}

	Flickable {
		id: flickable
		anchors.fill: parent
		flickableDirection: Flickable.VerticalFlick
		contentHeight: column.height+10

		Column {
			id: column
			width: parent.width
			height: childrenRect.height
			spacing: 20

			SectionSeperator { text: qsTr("STATE") }
			ListView {
				property int selectedSlot: /*StateListModel.InvalidSlot*/-0x80000000

				id: stateListView
				width: parent.width
				height: 280
				model: stateListModel
				spacing: 8
				orientation: ListView.Horizontal

				delegate: ImageListViewDelegate {
					id: stateDelegate
					width: 380
					height: 280
					imgSource: "image://state/" + slot + "*" + screenShotUpdate
					text: Qt.formatDateTime(saveDateTime, "dd.MM.yyyy hh:mm:ss")
					onPressed: stateListView.selectedSlot = slot
					onClicked: stateMenu.open()

					BorderImage {
						id: specialSlotImage
						source: "image://theme/meegotouch-video-duration-background"
						anchors {
							right: parent.right; rightMargin: 30
							top: parent.top; topMargin: 20
						}
						width: childrenRect.width+20
						height: childrenRect.height+10
						visible: slot < 0
						border { left: 10; right: 10 }

						Label {
							x: 10
							y: 5
							text: specialSlotImage.specialSlotName()
							color: "blue"
						}
						function specialSlotName() {
							if (slot == /*StateListModel.AutoSaveLoadSlot*/-2)
								return qsTr("AUTOSAVE")
							else if (slot == /*StateListModel.QuickSaveLoadSlot*/-3)
								return qsTr("QUICKSAVE")
							return slot
						}
					}

					ListView.onAdd: SequentialAnimation {
						PropertyAction { target: stateDelegate; property: "width"; value: 0 }
						NumberAnimation { target: stateDelegate; property: "width"; to: 380; duration: 250; easing.type: Easing.InOutQuad }
					}

					ListView.onRemove: SequentialAnimation {
						PropertyAction { target: stateDelegate; property: "ListView.delayRemove"; value: true }
						PauseAnimation { duration: 300 }
						NumberAnimation { target: stateDelegate; property: "width"; to: 0; duration: 250; easing.type: Easing.InOutQuad }
						PropertyAction { target: stateDelegate; property: "ListView.delayRemove"; value: false }
					}
				}
			}

			Column {
				id: inputColumn
				width: parent.width
				height: childrenRect.height
				spacing: 20

				SectionSeperator { text: qsTr("INPUT") }

				Label { text: qsTr("Pad Opacity"); x: 8 }
				Slider {
					id: padOpacitySlider
					width: parent.width
					minimumValue: 0.0
					maximumValue: 1.0
					stepSize: 0.05
					valueIndicatorVisible: true
					function refresh() { value = emuView.input.touch.opacity }
					onValueChanged: emuView.input.touch.opacity = value
					Component.onCompleted: refresh()
					Connections { target: emuView.input.touch; onOpacityChanged: padOpacitySlider.refresh() }
				}

				Label { text: qsTr("Port Index for Touch Screen"); x: 8 }
				ButtonRow {
					id: touchPadIndex
					platformStyle: ButtonStyle { }
					anchors.horizontalCenter: parent.horizontalCenter
					Button { id: padIndexA; property int padIndex: 0; text: qsTr("Port A") }
					Button { id: padIndexB; property int padIndex: 1; text: qsTr("Port B") }
					function refresh() {
						if (emuView.input.touch.padIndex === 1)
							padIndexB.checked = true
						else
							padIndexA.checked = true
					}
					onCheckedButtonChanged: emuView.input.touch.padIndex = checkedButton.padIndex
					Component.onCompleted: refresh()
					Connections { target: emuView.input.touch; onPadIndexChanged: touchPadIndex.refresh() }
				}

				Repeater {
					id: inputDevicesView
					model: emuView.input.devices

					SelectionItem {
						imageSource: qsTr("../img/input-%1.png").arg(modelData.shortName)
						titleText: modelData.name
						subtitleText: modelData.profile
						onClicked: {
							if (modelData.shortName == "touch") {
								touchProfileSelectionDialog.selectedProfile = modelData.profile
								touchProfileSelectionDialog.open()
							} else {
								inputDeviceProfilesModel.stringListModel = modelData.profiles
								inputDeviceProfileSelector.inputDevice = modelData
								inputDeviceProfileSelector.selectedIndex = modelData.profileIndex
								inputDeviceProfileSelector.open()
							}
						}
					}
				}
			}

			Column {
				id: videoColumn
				width: parent.width
				height: childrenRect.height
				spacing: 20

				SectionSeperator { text: qsTr("VIDEO") }

				Label { text: qsTr("Frameskip"); x: 8 }
				Slider {
					id: frameSkipSlider
					width: parent.width
					minimumValue: 0
					maximumValue: 5
					stepSize: 1
					valueIndicatorVisible: true
					function refresh() { value = emuView.runner.frameSkip }
					onValueChanged: emuView.runner.frameSkip = value
					Component.onCompleted: refresh()
					Connections { target: emuView.runner; onFrameSkipChanged: frameSkipSlider.refresh() }
				}

				EMSwitchOption {
					id: fpsVisbleSwitch
					text: qsTr("Show FPS")
					function refresh() { checked = emuView.video.fpsVisible }
					Component.onCompleted: refresh()
					onCheckedChanged: emuView.video.fpsVisible = checked
					Connections { target: emuView.video; onFpsVisibleChanged: fpsVisbleSwitch.refresh() }
					HelpLink { help: "Fps" }
				}

				EMSwitchOption {
					id: keepAspectRatioSwitch
					text: qsTr("Keep Aspect Ratio")
					function refresh() { checked = emuView.dmHelper.keepAspectRatio }
					Component.onCompleted: refresh()
					onCheckedChanged: emuView.dmHelper.keepAspectRatio = checked
					Connections { target: emuView.dmHelper; onKeepAspectRatioChanged: keepAspectRatioSwitch.refresh() }
				}

				SelectionItem {
					titleText: qsTr("Video Filter")
					subtitleText: emuView.video.filter
					onClicked: videoFilterSelector.open()
				}
				SelectionItem {
					titleText: qsTr("Orientation")
					subtitleText: orientationModel.get(emuView.dmHelper.orientation).name
					onClicked: orientationSelector.open()
				}
				PageSelectionItem {
					titleText: qsTr("Size and Position")
					subtitleText: qsTr("Configure display's size and position")
					onClicked: pageStack.push(Qt.resolvedUrl("../base/DisplayRectSelector.qml"),
											  { localSettings: true })
				}
				SelectionItem {
					titleText: qsTr("Video Profile")
					subtitleText: qsTr("Load Video Settings from Profile")
					onClicked: {
						videoProfileSelector.selectedIndex = -1
						videoProfileSelector.open()
					}
					visible: videoProfilesModel.count > 0
				}
			}

			Column {
				id: miscColumn
				width: parent.width
				height: childrenRect.height
				spacing: 20

				SectionSeperator { text: qsTr("MISC") }

				EMSwitchOption {
					id: audioEnabledSwitch
					text: qsTr("Audio Enabled")
					function refresh() { checked = emuView.audio.enabled }
					Component.onCompleted: refresh()
					onCheckedChanged: emuView.audio.enabled = checked
					Connections { target: emuView.audio; onEnabledChanged: audioEnabledSwitch.refresh() }
				}

				EMSwitchOption {
					id: benchmarkModeSwitch
					text: qsTr("Benchmark Mode")
					function refresh() { checked = emuView.runner.benchmarkMode }
					Component.onCompleted: refresh()
					onCheckedChanged: emuView.runner.benchmarkMode = checked
					Connections { target: emuView.runner; onBenchmarkModeChanged: benchmarkModeSwitch.refresh() }
				}

				SelectionItem {
					titleText: qsTr("Take Screenshot")
					subtitleText: qsTr("Overwrite Image in Disk Gallery")
					iconSource: "image://theme/icon-m-toolbar-captured-white"
					onClicked: {
						emuView.saveScreenShot()
						infoBanner.text = qsTr("Screenshot taken")
						infoBanner.show()
					}
				}
			}
		}

	}
	ScrollDecorator { flickableItem: flickable }

	Connections {
		target: emuView
		onFaultOccured: {
			emuFaultDialog.message = faultMessage
			emuFaultDialog.open()
		}
	}

	QueryDialog {
		id: yesNoDialog
		titleText: qsTr("Really?")
		acceptButtonText: qsTr("Yes")
		rejectButtonText: qsTr("No")
		onAccepted: settingsPage.executeAction(settingsPage.initiatedAction)
	}

	QueryDialog {
		id: emuFaultDialog
		titleText: qsTr("Oops")
		rejectButtonText: qsTr("Close")
	}
	StringListProxy {
		id: inputDeviceProfilesModel
	}
	SelectionDialog {
		property variant inputDevice
		id: inputDeviceProfileSelector
		model: inputDeviceProfilesModel
		titleText: qsTr("Select Profile")
		onAccepted: inputDevice.profile = inputDeviceProfilesModel.get(selectedIndex)
	}

	TouchProfileSelectionDialog {
		id: touchProfileSelectionDialog
		inputDevice: emuView.input.touch
		onAccepted: inputDevice.profile = selectedProfile
	}

	StringListProxy {
		id: videoFilterModel
		stringListModel: emuView.video.filters
	}
	SelectionDialog {
		id: videoFilterSelector
		model: videoFilterModel
		titleText: qsTr("Select Video Filter")
		onAccepted: emuView.video.filter = videoFilterModel.get(selectedIndex)
	}
	SelectionDialog {
		id: orientationSelector
		model: ListModel {
			id: orientationModel
			// TODO: use qsTr() in Qt5
			ListElement { name: "Landscape" }
			ListElement { name: "Portrait" }
			ListElement { name: "Landscape Inverted" }
			ListElement { name: "Portrait Inverted" }
		}
		titleText: qsTr("Select Orientation")
		onAccepted: emuView.dmHelper.orientation = selectedIndex

		Connections {
			target: emuView.dmHelper
			onOrientationChanged: orientationSelector.selectedIndex = emuView.dmHelper.orientation
		}
		Component.onCompleted: selectedIndex = emuView.dmHelper.orientation
	}

	StringListProxy {
		id: videoProfilesModel
		stringListModel: emuView.dmHelper.videoProfiles
	}
	SelectionDialog {
		id: videoProfileSelector
		model: videoProfilesModel
		titleText: qsTr("Select Video Profile")
		onAccepted: emuView.dmHelper.loadVideoProfile(videoProfilesModel.get(selectedIndex))
	}

	InfoBanner {
		id: infoBanner
		timerShowTime: 1500
	}
}
