/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

import QtQuick 1.1
import com.nokia.meego 1.1
import EmuMaster 1.0
import "../base"

Page {
	tools: ToolBarLayout {
		ToolIcon {
			iconId: "toolbar-back"
			onClicked: pageStack.pop()
		}
	}

	Flickable {
		id: flickable
		anchors.fill: parent
		flickableDirection: Flickable.VerticalFlick
		contentHeight: column.height+10

	Column {
		id: column
		width: parent.width
		height: childrenRect.height
		spacing: 20

		SectionSeperator { text: qsTr("STATE") }

		GlobalSettingsSwitchItem { text: qsTr("Auto Load/Save on Start/Exit"); optionName: "autoSaveLoadEnable" }

		SectionSeperator { text: qsTr("INPUT") }

		PageSelectionItem {
			titleText: qsTr("Touch Configuration")
			imageSource: "../img/input-touch.png"
			onClicked: pageStack.push(Qt.resolvedUrl("TouchSettingsPage.qml"))
		}
		PageSelectionItem {
			titleText: qsTr("Keyboard Configuration")
			imageSource: "../img/input-keyb.png"
			onClicked: pageStack.push(Qt.resolvedUrl("KeybMappingPage.qml"))
		}
		PageSelectionItem {
			titleText: qsTr("Calibrate Accelerometer")
			imageSource: "../img/input-accel.png"
			onClicked: pageStack.push(Qt.resolvedUrl("AccelCalibrationPage.qml"))
		}

		SectionSeperator { text: qsTr("VIDEO") }

		Label { text: qsTr("Frameskip"); x: 8 }
		Slider {
			width: parent.width
			minimumValue: 0
			maximumValue: 5
			stepSize: 1
			valueIndicatorVisible: true
			onValueChanged: diskGallery.setGlobalOption("frameSkip", value)
			Component.onCompleted: value = diskGallery.globalOption("frameSkip")
		}
		GlobalSettingsSwitchItem {
			text: qsTr("Show FPS")
			optionName: "fpsVisible"
			HelpLink { help: "Fps" }
		}
		GlobalSettingsSwitchItem {
			id: keepAspectRatioSwitch
			text: qsTr("Keep Aspect Ratio")
			optionName: "keepAspectRatio"
			HelpLink { help: "KeepAspectRatio" }
		}

		SelectionItem {
			titleText: qsTr("Orientation")
			subtitleText: orientationModel.get(orientationSelector.selectedIndex).name
			onClicked: orientationSelector.open()
		}

		PageSelectionItem {
			titleText: qsTr("Size and Position")
			subtitleText: qsTr("Configure display's size and position")
			onClicked: pageStack.push(Qt.resolvedUrl("../base/DisplayRectSelector.qml"),
									  { localSettings: false })
		}

		Label { text: qsTr("Video Profile"); x: 8 }

		ButtonRow {
			x: 8
			exclusive: false

			Button {
				text: qsTr("Load")
				onClicked: {
					videoProfileSelector.selectedIndex = -1
					videoProfileSelector.loadDeleteSwitch = false
					videoProfileSelector.open()
				}
			}
			Button {
				text: qsTr("Save")
				onClicked: videoProfileNameDialog.open()
			}
			Button {
				text: qsTr("Delete")
				onClicked: {
					videoProfileSelector.selectedIndex = -1
					videoProfileSelector.loadDeleteSwitch = true
					videoProfileSelector.open()
				}
			}
		}

		SectionSeperator { text: qsTr("MISC") }

		GlobalSettingsSwitchItem { text: qsTr("Audio Enabled"); optionName: "audioEnable" }
		GlobalSettingsSwitchItem { text: qsTr("Extended Vol. Keys Actions"); optionName: "extHwButtonsActions" }
		GlobalSettingsSwitchItem { text: qsTr("L3/R3 as Quick Save/Load"); optionName: "L3R3AsQuickSaveLoad" }
		GlobalSettingsSwitchItem { text: qsTr("Swipe Enabled"); optionName: "swipeEnable" }
		GlobalSettingsSwitchItem { text: qsTr("New Framerate Counter"); optionName: "fpsEmulated" }

		EMSwitchOption {
			id: runInBackgroundSwitch
			text: qsTr("Run in Background")
			checked: diskGallery.globalOption("runInBackground")
			onCheckedChanged: {
				if (checked) {
					// special handling to prevent the dialog from showing at the start
					if (diskGallery.globalOption("runInBackground") !== "true")
						runInBackgroundDialog.open()
				} else {
					diskGallery.setGlobalOption("runInBackground", false)
				}
			}
		}

		SelectionItem {
			titleText: qsTr("Default PSX BIOS")
			subtitleText: psxBiosDialog.selectedBios()
			onClicked: psxBiosDialog.open()
		}
	}

	}
	ScrollDecorator { flickableItem: flickable }

	QueryDialog {
		id: runInBackgroundDialog
		titleText: qsTr("Enabling the option will run the application in the background. " +
						"This can consume a lot of battery!")
		acceptButtonText: qsTr("Enable")
		rejectButtonText: qsTr("Disable")
		onRejected: runInBackgroundSwitch.checked = false
		onAccepted: diskGallery.setGlobalOption("runInBackground", true)
	}

	SelectionDialog {
		id: orientationSelector
		model: ListModel {
			id: orientationModel
			// TODO: use qsTr() in Qt5
			ListElement { name: "Landscape" }
			ListElement { name: "Portrait" }
			ListElement { name: "Landscape Inverted" }
			ListElement { name: "Portrait Inverted" }
		}
		titleText: qsTr("Select Orientation")
		onAccepted: diskGallery.setGlobalOption("orientation", selectedIndex)
		Component.onCompleted: selectedIndex = diskGallery.globalOption("orientation")
	}

	StringListProxy {
		id: videoProfilesModel
		stringListModel: diskGallery.videoProfiles
	}
	SelectionDialog {
		id: videoProfileSelector
		property bool loadDeleteSwitch: false
		model: videoProfilesModel
		titleText: qsTr("Select Video Profile")
		onAccepted: {
			var name = videoProfilesModel.get(selectedIndex)
			if (loadDeleteSwitch)
				diskGallery.removeVideoProfile(name)
			else
				diskGallery.loadVideoProfile(name)
		}
	}

	Connections {
		target: diskGallery
		onVideoProfileLoaded: {
			keepAspectRatioSwitch.checked = diskGallery.globalOption("keepAspectRatio")
			orientationSelector.selectedIndex = diskGallery.globalOption("orientation")
		}
	}

	Dialog {
		id: videoProfileNameDialog
		content: Column {
			width: videoProfileNameDialog.width
			spacing: 16

			Label {
				anchors.horizontalCenter: parent.horizontalCenter
				text: qsTr("Type name of the profile to be saved:")
			}
			TextField {
				id: textField
				anchors.horizontalCenter: parent.horizontalCenter
			}

			Button {
				text: qsTr("OK")
				onClicked: {
					if (textField.text != "") {
						diskGallery.saveVideoProfile(textField.text)
						videoProfileNameDialog.accept()
					}
				}
			}
			Button {
				text: qsTr("Cancel")
				onClicked: videoProfileNameDialog.reject()
			}
		}
	}

	SelectionDialog {
		id: psxBiosDialog
		property string psxBiosOption: "psx.bios"
		titleText: qsTr("Select BIOS")
		model: StringListProxy {
			id: psxBiosModel
			stringListModel: diskListModel.psxBioses
		}
		function selectedBios() { return (selectedIndex>=0) ? psxBiosModel.get(psxBiosDialog.selectedIndex) : "<Invalid>" }
		Component.onCompleted: selectedIndex = psxBiosModel.indexOf(diskGallery.globalOption(psxBiosOption))
		onAccepted: diskGallery.setGlobalOption(psxBiosOption, selectedBios())
	}
}
